# updateUserNames.py - Updates the usernames after an AD change.
#                      The script reads old and new users name from FILE and renames
#                      the users accordingly.
#
# Copyright (c) 2012, 2013, 2014, 2015 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# Modify this according to your installation
# Administration API has to be enabled
USER = "admin"
PASS = "admin"
HOST = "localhost"
PORT = 8474
IGNORESSLWARNINGS = True

INSTANCEID = "test"

# The file which contains old and new usernames.
# By default MailStore takes sAMAaccountName as "primary key" (can be configured in the sync-settings),
# so usually you need to enter this as newname.
# You can check this, with the "Test settings" option in MailStore Client.
# One user per line, format:
# oldname=newname
# Comments in the file start with # or ;
user_file_path = "users.txt"

# Enter "rename" to to rename the users according to FILE.
# Enter "prepare" to read current users from MailStore and prepare FILE.
# operation = "rename"
operation = "prepare"

# ################## DON'T CHANGE ANYTHING BELOW THIS LINE ###################
import mailstore


# Callback function for the API
def print_log_output(values):
    print(values, flush=True)


api = mailstore.mgmt.SPEClient(USER, PASS, HOST, PORT, callback=print_log_output,
                               ignoreInvalidSSLCerts=IGNORESSLWARNINGS, instanceID=INSTANCEID)


def modifyUsers():
    with open(user_file_path) as user_file:
        for line in [lines for lines in user_file.readlines() if not lines.startswith(("#", ";"))]:
            old_username, new_username = [l.strip() for l in line.split("=")]
            get_users_result = api.GetUsers()["result"]
            current_users = [user["userName"] for user in get_users_result]
            if (old_username in current_users) and (old_username != new_username):
                # Rename user
                if new_username not in current_users:
                    print("### renaming user {} to {} ".format(old_username, new_username))
                    rename_user_result = api.RenameUser(old_username, new_username)
                    # If renaming was successful, perform other actions
                    if rename_user_result["statusCode"] == "succeeded":
                        # Move folder
                        print("move_folder {} -> {}".format(old_username, new_username))
                        print("+---------------------------------")
                        print(api.MoveFolder(old_username, new_username))
                        print("+---------------------------------")
                        # Get privileges on own folder
                        get_user_info_result = api.GetUserInfo(INSTANCEID, new_username)["result"]
                        if "privilegesOnFolders" in get_user_info_result:
                            for folder in get_user_info_result["privilegesOnFolders"]:
                                if folder["folder"] == old_username:
                                    privileges = folder["privileges"]
                                    print("privileges for user {0} on {0} were {1}".format(old_username, privileges))
                                    break
                            else:  # no break, if no privilege on own folder was found, set default privilege
                                privileges = ["none"]

                            # Granting new privileges
                            print("setting privilege for user {0} on folder {0} to {1} ".format(new_username,
                                                                                                privileges), end="")
                            print(api.SetUserPrivilegesOnFolder(new_username, new_username, privileges))

                            # Revoking old privilege
                            print("setting privilege for user {} on folder {} to 'none' ".format(new_username,
                                                                                                 old_username), end="")
                            print(api.SetUserPrivilegesOnFolder(new_username, old_username, "none"))
                    else:
                        print("renaming user was not successful, no other actions on this user will be performed")


def prepare():
    with open(user_file_path, mode="w") as user_file:
        for user in api.GetUsers()["result"]:
            print("user {}".format(user))
            user_file.write("{}=\n".format(user["userName"]))


if __name__ == "__main__":
    if operation == "prepare":
        prepare()
    elif operation == "rename":
        modifyUsers()
    else:
        print("operation '{}' not permitted".format(operation))
    print("done")
